
import altair as alt
from .distributional import hist
from .relational import scatterplot
from .interactions import Interaction, apply_effect, process_effects

def jointplot(data=None,x=None, y=None,effects=None,width=200,height=200):
  """
    Return the most important thing about a person.
    Parameters
    ----------
    data
        A string indicating the name of the person.
  """
   
  top = hist(data=data,x=x,width=200,height=50,xAxis=None,yAxis=None)
  right = hist(data=data,y=y,width=50,height=200,xAxis=None,yAxis=None)

  mid = scatterplot(data=data,x=x,y=y)


  # question is there a way to 
  chart =  alt.vconcat(top, alt.hconcat(mid,right,spacing=-10), spacing=-10)

  if effects:
    chart = process_effects(chart,effects)

  return chart.properties(width=width,height=height)




import altair as alt

from altair_express.interactions import process_effects
from .utils import data_type_converter, create_dataframe
import pandas as pd
from .distributional import heatmap

def relplot(data=None, *, x=None, y=None,color=None,interactive=None,kind="scatter",width=200,height=200):

  if kind == "scatter":
    return scatterplot(data=data,x=x,y=y,color=color,interactive=interactive,width=width,height=height)
  elif kind == "line":
    return lineplot(data=data,x=x,y=y,color=color,interactive=interactive,width=width,height=height)
  else : 
    raise ValueError('[relplot] kind parameter should be one of "scatter" or "line"')

def generate_shorthand(input_field,data):
  # if input_field contains ":Q"  or ":N" then return input_field
  if input_field[-2:] == ":Q" or input_field[-2:] == ":N" or input_field[-2:] == ":O":
    return input_field # already as shorthand

  data_type = data_type_converter(data.dtypes[input_field])
  return f'{input_field}:{data_type}'


def barplot(data=None, x=None, y=None,y_axis=alt.Axis(),x_axis=alt.Axis(),color=None,column=None,effects=None,width=200,height=200):
  data, x, y = create_dataframe(data=data,x=x,y=y)

  params = {}
  facet_params = {}
  if color:
     params['color'] = color
      
  #if column:
  #    facet_params['column'] = column
  #    params['column'] = column
  if column:
    num_facets = len(pd.unique(data[column]))
    width = width / num_facets

  chart = alt.Chart(data).mark_bar().encode(
    alt.X(x,axis=x_axis),
    alt.Y(y,axis=y_axis),
    **params
  ).properties(width=width,height=height)
  # properties is placed up here so that it is captured by the facet

  if column:
    chart = chart.facet(column=column)
  
  if effects:
    chart = process_effects(chart,effects)

  return chart

def lineplot(data=None, x=None, y=None,y_axis=alt.Axis(),x_axis=alt.Axis(),color=None,effects=None,width=200,height=200):
  data, x, y = create_dataframe(data=data,x=x,y=y)

  fill = 'steelblue'
  
  chart = alt.Chart(data).mark_line(fill=fill).encode(
    alt.X(x, scale=alt.Scale(zero=False),axis=x_axis),
    alt.Y(y, scale=alt.Scale(zero=False),axis=y_axis),
  )

  if color:
    if color not in data.columns:
        fill = color
    else:
        chart=chart.encode(alt.Color(field=color,scale=alt.Scale()))
  
  if effects:
    chart = process_effects(chart,effects)

  return chart.properties(width=width,height=height)

def kdeplot(data=None,x=None,color=None,effects=None,width=200,height=200):
  x_shorthand = generate_shorthand(x,data)

  line_color = 'steelblue'
  transform_params ={
    "density":x,
    "as_":[x,'density'],
    "groupby":[color],
  }
  not_none_params = {k:v for k, v in transform_params.items() if v is not None and (not isinstance(v,list) or len(v)>0 and v[0] is not None)}

  chart = alt.Chart(data).mark_line().transform_density(**not_none_params).encode(
    alt.X(shorthand=x_shorthand, scale=alt.Scale(zero=False)),
    alt.Y(shorthand='density:Q'),
  )

  if color:
    if color not in data.columns:
        chart= chart.mark_line(fill=line_color)
    else:
        chart=chart.encode(alt.Color(field=color,scale=alt.Scale()))

  if effects:
    chart = process_effects(chart,effects)

  return chart

def scatterplot(data=None, *, x=None, y=None,xAxis=alt.Axis(),color=None,yAxis=alt.Axis(),effects=None,fill="steelblue",width=200,height=200):

  data, x, y = create_dataframe(data=data,x=x,y=y)

  x_type = data_type_converter(data.dtypes[x])
  y_type = data_type_converter(data.dtypes[y])

  
  chart = alt.Chart(data).mark_circle().encode(
      alt.X(shorthand=f'{x}:{x_type}', scale=alt.Scale(zero=False),axis=xAxis),
      alt.Y(shorthand=f'{y}:{y_type}', scale=alt.Scale(zero=False),axis=yAxis),
  )

  if color:
    if color not in data.columns:
        chart=chart.mark_circle(fill=color)
    else:
        chart=chart.encode(alt.Color(field=color))

  if effects:
    chart = process_effects(chart,effects)
  


  return chart.properties(width=width,height=height)


def corr_map(data=None, vars=None):
  if vars is None:
    vars = data.columns

  correlation = data.corr()
  return heatmap(correlation)


"""Top-level package for Altair Express."""

__author__ = """Dylan Wootton"""
__email__ = 'dwootton@mit.edu'
__version__ = '0.1.24'

from .jointplot import *
from .pairplot import *
from .distributional import *
from .relational import *
from .data_table import *
from .interactions import *



import altair as alt
from .distributional import hist
from .relational import scatterplot
import numpy as np

def pairplot(data=None,variables=None):
  if data is None:
    raise ValueError('[pairgrid] data cannot be null')

  if variables is None:
    # get all numeric columns from the dataframe 
    variables = list(data.select_dtypes(include=[np.number]).columns.values)

  grid = []
  
  for row_index,row_variable in enumerate(variables):
    grid_row = []
    for column_index,column_variable in enumerate(variables):
      chart = None
      xAxis = None
      yAxis = None

      if row_index == len(variables)-1 :
         xAxis = alt.Axis()

      if column_index == 0:
         yAxis = alt.Axis()

      
      if row_variable == column_variable:
       
        # ,
        chart = hist(data,x=column_variable,yAxis=yAxis,xAxis=xAxis).properties(height=100,width=100)
        if row_index == 0:
          chart.encoding.y.title = row_variable
        if row_index == len(variables)-1:
          chart.encoding.x.title = row_variable
        
      else:
       
        chart = scatterplot(data,x=column_variable,y=row_variable,yAxis=yAxis,xAxis=xAxis).properties(height=100,width=100)
      grid_row.append(chart)

    grid.append(grid_row)

  chart = None
  rows =[]

  for row_index in range(len(grid)):
      rows.append(alt.hconcat(*grid[row_index],spacing=5))

  chart = alt.vconcat(*rows,spacing=5)
    
  return chart